<?php

/*
   ----------------------------------------------------------------------
Standard Deviation Calculator

Language: PHP v7.1.8
Author: Jay Tanner - 2018
www.NeoProgrammics.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.


This program computes the sum, the arithmetic mean, the standard deviation
and the population and sample variances for a given set of numerical data
representing a population or sample derived therefrom.

Simply copy/paste the numerical data into the text input area and the
program does all the number-crunching work. The numerical data can be
delimited by spaces, commas or semicolons.  Any number of data points
are allowed.

--------------------------------------------------------------------
*/



// Initialize for cookies
   ob_start();

   $cYear = gmdate("Y");

   $_PROGRAM_VERSION_  = "v2.21";

   $n = $SUMdata = $mean = $SUMsquaresOfDiffs = $iInit = 0;
   $variance0 = $variance1 = $StdDev0 = $StdDev1 = 0;
   $WorkString = '';

// Local server path to this PHP program script.
// This script calls itself and self-modifies
// when the [COMPUTE] button is clicked.
   $_RUN_THIS_PROGRAM_ = $_SERVER['SCRIPT_NAME'];

   $_HTML_TITLE_ = "Standard Deviation and Variance Statistics Calculator - ";
   $_HEADER_     = "Standard Deviation and Variance Statistics Calculator";

// Give the cookie a name.  This cookie data can be shared
// with any other scripts running within the same folder.
// SPECIAL CASE NOTE: This program uses no cookie data.
// It creates a cookie with the string: 'Empty cookie.'
   $_COOKIE_NAME_ = "Std_Dev_Interface_Settings";

// ===========================================================
// Set cookie expiration date to 1 standard month (30 days).
//
// This value must be the number of seconds as measured from
// the moment the cookie is refreshed. To set the cookie to
// expire in 30 days, simply multiply the number of days
// by 86400 seconds per day.  Each time the cookie is
// used, it will be refreshed. It will expire only if
// called later than 30 days from the last time it
// was refreshed.

   $ExpiresIn30Days = time() + (30 * 86400);

// =======================================
// Do this if "COMPUTE" button was clicked.

   if (IsSet($_POST["COMPUTE_Button"]))
  {
// Read interface input text area.
   $NumericDataString = HTMLentities(trim(@$_POST["NumericDataString"]),ENT_QUOTES,'ISO-8859-1');

   $w = str_replace(',',  ' ', $NumericDataString);
   $w = str_replace(';',  ' ', $w);
   $w = str_replace('\n', ' ', $w);
   $WorkString = preg_replace('/\s+/', ' ', trim($w));

// Write/save interface settings to cookie.  In this
// case, no cookie data are used.
   $CookieDataString = '';

   setcookie ($_COOKIE_NAME_, $CookieDataString, $ExpiresIn30Days);
  }

   else

// ======================================================
// Do this if program was called externally via URL link.

      {

// =========================================================
// If a cookie exists, then recall interface value from it.

   if (isset($_COOKIE[$_COOKIE_NAME_]))
  {
// Recall interface text field settings from cookie.
   $CookieDataString = $_COOKIE[$_COOKIE_NAME_];

// Parse cookie data and get values into their respective variables.
// SPECIAL CASE NOTE: This program uses no cookie data.
// list($NumericDataString) = preg_split("[\|]", $CookieDataString);
   $WorkString = $NumericDataString = '';
  }

   else

// ===============================================
// If no cookie exists, then set default interface
// values and copy to an initial cookie.

  {
// Set initial random default/test data values.
   if (@$WorkString == '')
      {
       $R = mt_rand(10, 50);
       $rd = mt_rand(0, 5);
       for($r=0;   $r < $R;   $r++)
          {
           $rn = mt_rand(500, 505) + floatval(Random_Decimals($rd));
           $WorkString .= "$rn, ";
          }
      }
   $WorkString = rtrim(trim($WorkString), ',');
   $CookieDataString = '';
   $NumericDataString = $WorkString;

   setcookie ($_COOKIE_NAME_, $CookieDataString, $ExpiresIn30Days);
  }
}



// **********************************
// **********************************
// AT THIS POINT, THE INPUT INTERFACE
// VALUES SHOULD BE READY FOR USE




// -------------------------------------------
// THIS IS WHERE THE INPUT ERROR FILTER(S) GO.
// DEFAULT IS NO ERROR CHECKING (0==0).



   if (strpos(trim($WorkString), ' ') === FALSE)
  {
   $out =
"
ENTER a data string or table above with at least two numbers to be evaluated.

The data are given as a space-delimited numeric string.  However, the data can
also be delimited by commas or semicolons as well.
";
  }

   else

/*
  ============================
  CONTINUE BELOW IF NO ERRORS.
  MAIN COMPUTATIONS START HERE. THE STATISTICS RETURNED ARE:
  n = Data count
  SUMdata = Sum of all data
  mean = Arithmetic mean of data
  SUMsquaresOfDiffs = Sum of squares of all differences from the mean
  variance0 = Population variance
  StdDev0
  variance1
  StdDev1
*/

  {
// Call function to compute the standard deviation and variance statistics.
   $StdDevStatsStr = Std_Dev_Stats ($WorkString);

   list($n,$SUMdata,$mean,$SUMsquaresOfDiffs,$variance0,$StdDev0,$variance1,$StdDev1) =
   preg_split("[ ]", $StdDevStatsStr);

// If n > 0, the set initial i=1
   $iInit  = ($n > 0)? 1:0;


// ======================================
// CONSTRUCT FORMATTED TEXT OUTPUT BLOCK.
//
// After the computations are done, the raw numeric results can be
// formatted for text output here, usually in a PRE block to make
// sure the text output is neatly aligned.

// Format computed results for neat output display.
// $n $SUMdata $mean $SUMsquaresOfDiffs $variance0 $StdDev0 $variance1 $StdDev1




   $out =
"
n = Data count = $n
Sum of all data = $SUMdata
Arithmetic mean of data = $mean
Sum of squares of all differences from the mean = $SUMsquaresOfDiffs
Population variance = $variance0
Population standard deviation = $StdDev0
Sample variance = $variance1
Sample standard deviation = $StdDev1
";



  }





/*
  ------------------------------------------------------------------------------
  THIS FUNCTION IS THE FOUNDATION UPON WHICH THIS PROGRAM IS BUILT.
  REVISED VERSION AS OF: 2014 AUG 30
  OUTPUT SEQUENCE IS DIFFERENT THAN EARLIER VERSIONS AND MAY CHANGE
  ADDING MORE STATISTICS IN FUTURE VERSIONS.

  This function computes the standard statistics for a numerical data string.

  The returned string contains the following computed statistics:
  n                  = Number of data elements (numbers to be processed)
  $SUMdata           = Sum of all the data
  $mean              = Arithmetic mean of the data
  $SUMsquaresOfDiffs = Sum of the squares of the differences from the mean
  $variance0         = Population variance
  $StdDev0           = Population standard deviation
  $variance1         = Sample variance
  $StdDev1           = Sample standard deviation"
*/

   function Std_Dev_Stats ($NumericDataString)
{
   $w = str_replace(',',  ' ', $NumericDataString);
   $w = str_replace(';',  ' ', $w);
   $w = str_replace('\n', ' ', $w);
   $WorkString = preg_replace('/\s+/', ' ', trim($w));

   $n = $SUMdata = 0;

   while ($WorkString != '')
  {
   $n++;

   $i = strpos($WorkString, ' ');

   if ($i === FALSE)
      {
       $SUMdata += $WorkString;
       $mean = $SUMdata / $n;
       break;
      }

   $xi = trim(substr($WorkString, 0, $i));
   $SUMdata += $xi;
   $mean = $SUMdata / $n;

   $WorkString = trim(substr($WorkString, $i, strlen($WorkString)));
  }

// *************************************************************************
// Cycle through data computing the sum of squares of differences from mean.
// The mean of this summation of squares is the population variance.

   $w = str_replace(",", " ", $NumericDataString);
   $w = str_replace(";", " ", $w);
   $WorkString = preg_replace('/\s+/', ' ', trim($w));

   $n = $SUMsquaresOfDiffs = 0;

   while ($WorkString != '')
  {
   $n++;

   $i = strpos($WorkString, ' ');

   if ($i === FALSE)
      {
       $diff = $WorkString - $mean;
       $SUMsquaresOfDiffs += ($diff * $diff);
       break;
      }

   $xi = trim(substr($WorkString, 0, $i));
   $diff = $xi - $mean;
   $SUMsquaresOfDiffs += ($diff * $diff);
   $WorkString = trim(substr($WorkString, $i, strlen($WorkString)));
  }

   $variance0 = $SUMsquaresOfDiffs / ($n-0); // sigma
   $StdDev0 = sqrt($variance0); // SqRt(sigma)

   $variance1 = $SUMsquaresOfDiffs / ($n-1); // s
   $StdDev1 = sqrt($variance1); // SqRt(s)

   $StdDevStats = "$n $SUMdata $mean $SUMsquaresOfDiffs $variance0 $StdDev0 $variance1 $StdDev1";

   return $StdDevStats;

} // End of  Std_Dev_Stats(...)



/*
   ========================================================================
   This function generates a string of any number of
   random digits in any given base from 2 to 36.

   All letters (A to Z) used as digits will be returned in uppercase.

   ERRORS:
   On error, FALSE is returned for invalid argument(s).
   ========================================================================
*/

   function Random_Digits_BaseB ($NumDigits=1, $BaseB=10, $LZeroSuppressed=FALSE)
{

// Read arguments.
   $N = intval($NumDigits);
   $B = intval($BaseB);

// Check for invalid argument(s).
   if ($B < 2 or $B > 36 or $N < 1) {return FALSE;}

// Define digits spectrum for the given base.
   $DigitsSpectrum = substr("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ",0,$B);

// Construct an N-digit random sequence in base B, one digit at
// a time, using a random 50/50 digit alternation method. There
// will be at least one random base B digit returned.
   $RandomDigits = "";

   for ($i=0;   $i < $N;   $i++)
  {
   if (mt_rand() % 2 == 0)
  {$RandomDigits .= substr($DigitsSpectrum, mt_rand(0,$B-1),1);}
   else
  {$RandomDigits = substr($DigitsSpectrum, mt_rand(0,$B-1),1) . $RandomDigits;}
  }

// If LZeroSuppressed flag === TRUE, then leftmost digit of returned
// random digits string must NOT be zero.  If it is, then another
// random non-zero digit will be attached to the left of it and
// the rightmost digit dropped.  The result will be a normal
// integer value with the required number of digits, but
// never starting with 0 (zero) as the first digit.
   if ($LZeroSuppressed === TRUE)
  {
   $RandomDigits = substr($DigitsSpectrum, mt_rand(1,$B-1),1) . $RandomDigits;
  }
   return substr("$RandomDigits", 0, $N);

} // End of  Random_Digits_BaseB(...)



// Special custom routine to add an d-digit random decimal value.

   function Random_Decimals($NumDecimals)
{
   if ($NumDecimals == 0) {return '';};

   return '.' . Random_Digits_BaseB ($NumDecimals);
}





















// Set number of output TEXTAREA rows according
// to the length of the output string.

   $TextOutRows = 5 + floor(strlen($out) / 80);


// =========================================================
// DYNAMIC W3C COMPLIANT HTML DOCUMENT OUTPUT FOLLOWS BELOW.

print <<< _HTML

<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">

<html>

<head>

<meta http-equiv="content-type" content="text/html; charset=UTF-8">

<title>$_HTML_TITLE_ - PHP Science Labs</title>

<meta name="robots"    content="noindex,nofollow">
<meta name="googlebot" content="noindex,nofollow">

<style type='text/css'>
BODY
{color:silver; background-color:black;  font-family:Verdana; font-size:15px}

TABLE
{font-size:12px; border: 1px solid black;}

TD
{color:black; background-color:white;  line-height:150%; font-size:18px; padding:6px; text-align:center;}

DIV
{color:black; background-color:white;  font-family:Verdana; text-align:justify; padding:4px; font-size:16px;}

PRE
{color:black; background-color:white;  font-family:monospace; font-size:14px; font-weight:bold; text-align:left; padding:4px;}

CODE
{font-family:monospace; font-size:16px; font-weight:bold;}

TEXTAREA
{background-color:LightYellow; color:black; font-family:monospace; font-size:18px; font-weight:bold; padding:4px; border:1px solid black; border-radius:8px;}

IMG
{box-shadow:2px 2px 3px #666666; border:1px solid silver;}

INPUT[type="text"]::-ms-clear {width:0; height:0;}

INPUT[type="text"]
{color:black; background-color:white; font-size:18px; font-family:monospace; font-weight:bold; text-align:center; box-shadow:2px 2px 3px #666666; border:2px solid black;}

INPUT[type="text"]:focus
{background-color:white; box-shadow:2px 2px 3px #666666; font-family:monospace; font-size:18px; border:2px solid red; text-align:center; font-weight:bold;}

INPUT[type="submit"]
{font-weight:bold;}

INPUT[type="submit"]:hover
{box-shadow:2px 2px 5px #222222; font-weight:bold;}

INPUT[type="checkbox"]:checked
{box-shadow:2px 2px 3px #222222;}

HR {background-color:black; height:2px; border:0px;}

A:link,  A:visited {background-color:transparent; color:blue; font-family:Verdana; font-size:90%; font-weight:bold; text-decoration:none; line-height:150%; padding:2px;}
A:hover, A:active  {background-color:transparent; color:red;  font-family:Verdana; font-size:90%; font-weight:bold; text-decoration:none; line-height:150%; padding:2px;}

::selection      {background-color:yellow; color:black;}  /* Only affects IE9+ and Safari     */
::-moz-selection {background-color:yellow; color:black;}  /* Only affects FireFox and Mozilla */
</style>

</head>

<body style='background-color:black;'>

<table width='530'>
<tr><td style='color:white; background-color:#0000DD; font-size:16px; border:2px solid white; border-radius:8px 8px 0px 0px; text-align:center;'><b>$_HEADER_<br></td></tr>

<tr><td style='border:1px solid black; font-family:verdana; font-size:12px;'>
<br>
<b>Simply enter the delimited numerical data string/table and click [COMPUTE].<br>
The numerical data elements can be separated by commas, spaces or semicolons.
<br><br>
The initial example data is randomly generated.&nbsp; Replace with your own data table.</b>

<br><br>
<form name="form2" method="post"  action="$_RUN_THIS_PROGRAM_" style='background-color:LightYellow; text-align:center; padding:4px; border:1px solid black;'>
<br>
<textarea name='NumericDataString' style="background-color:white; border:1px solid black; box-shadow:3px 2px 6px #666666;" cols="60" rows="10">
$NumericDataString
</textarea>
<br><br>
<input name="COMPUTE_Button" type="submit" value="C O M P U T E"><br><br>
</form>

<br>

<!-- Start table within a table -->
<table width='530' align='center' cellpadding='8' cellspacing='1' style='background-color:black; font-family:monospace; font-size:16px;'>
<tr><td colspan='2' style='background-color:yellow; font-weight:bold;'>Statistics Computed From The Data Given Above</td></tr>
<tr><td style='background-color:black; color:white; font-weight:bold;'>Mathematical&nbsp;Expression</td><td align='center' style='background-color:GreenYellow; font-weight:bold;'>&nbsp;Numerical&nbsp;Evaluation&nbsp;</td></tr>
<tr><td><img src='images/N_cap.png' alt='' style='border:none; box-shadow:none;'><br>Data count<br></td><td align='center' style='font-weight:bold;'>$n</td></tr>
<tr><td><img src='images/i.png' alt='' style='border:none; box-shadow:none;'><br>Data index<br></td><td align='center'style='font-weight:bold;'>$iInit to $n</td></tr>

<tr><td><img src='images/sum_of_all_data.png' alt=''><br>Sum of all given data</td><td align='center' style='font-weight:bold;'>$SUMdata</td></tr>
<tr><td><img src='images/population_or_sample_mean_mu_bar_x.png' alt=''><br>Arithmetic mean of population or sample</td><td align='center' style='font-weight:bold;'>$mean</td></tr>
<tr><td><img src='images/sum_of_squares_of_diffs.png' alt=''><br>&nbsp;Sum&nbsp;of&nbsp;squares&nbsp;of&nbsp;all&nbsp;differences&nbsp;from&nbsp;mean&nbsp;</td><td align='center' style='font-weight:bold;'>$SUMsquaresOfDiffs</td></tr>
<tr><td><img src='images/population_variance.png' alt=''><br>Population variance</td><td align='center' style='font-weight:bold;'>$variance0</td></tr>
<tr><td><img src='images/population_std_dev.png' alt=''><br>Population standard deviation</td><td align='center' style='font-weight:bold;'>&plusmn; $StdDev0</td></tr>
<tr><td><img src='images/sample_variance.png' alt=''><br>Sample variance</td><td align='center' style='font-weight:bold;'>$variance1</td></tr>
<tr><td><img src='images/sample_std_dev.png' alt=''><br>Sample standard deviation</td><td align='center' style='font-weight:bold;'>&plusmn; $StdDev1</td></tr>
<tr><td colspan='2' style='font-size:12px;'>Program by Jay Tanner - $cYear - $_PROGRAM_VERSION_</td></tr>
</table>
<!-- End table within a table -->

</td></tr></table>

</body>
</html>


_HTML;


/*
   Optional PHP code can go here.
*/





?>
